class Button
	attr_reader :part

	def initialize(collection, name, rel, image, x, y)
		@collection = collection
		@name = name
		@rel = rel
		@image = image
		@x = x
		@y = y

		@part = setup_part
		setup_programs
	end

	private
	def setup_part
		ret = nil

		@collection.part(@name, :image) do |p|
			ret = p

			["normal", "hover", "down"].each do |state|
				s = state == "normal" ? "default" : state

				p.description(s) do |d|
					d.rel[0].to = d.rel[1].to = @rel
					d.rel[0].set_offset(@x, @y)

					d.auto_rel = true
					d.image = "#{@image}-#{state}.png"
				end
			end
		end

		ret
	end

	def setup_programs
		{"in" => "hover", "out" => "default"}.each do |sig, state|
			@collection.program("#{@name}.#{sig}",
			                    :set_state) do |p|
				p.signal = "mouse,#{sig}"
				p.source = @name
				p.state = state
				p.time = 0.2
				p.targets << ProgramTarget.new(@part)
			end
		end

		{"up" => "hover", "down" => "down"}.each do |sig, state|
			@collection.program("#{@name}.#{sig}",
			                    :set_state) do |p|
				p.signal = "mouse,#{sig},1"
				p.source = @name
				p.state = state
				p.targets << ProgramTarget.new(@part)
			end
		end
	end
end

class SignalButton < Button
	def initialize(collection, name, rel, image, x, y, signal)
		super(collection, name, rel, image, x, y)

		collection.program("#{name}.clicked",
		                    :emit_signal) do |p|
			p.signal = "mouse,clicked,1"
			p.source = name
			p.emission_signal = signal
			p.emission_source = name
		end
	end
end

class ButtonLed
	def initialize(c, name, rel, image, x, y)
		@c = c
		@name = name
		@rel = rel
		@image = image
		@x = x
		@y = y

		@part = setup_part
		setup_programs
	end

	private
	def setup_part
		ret = nil

		@c.part(@name, :image) do |p|
			ret = p
			p.mouse_events = false

			p.description("default") do |d|
				d.visible = false
				d.color = "#ffffff00"

				d.rel[0].to = d.rel[1].to = @rel
				d.rel[0].set_offset(@x, @y)

				d.auto_rel = true
				d.image = "#{@image}.png"
			end

			p.description("hover") do |d|
				d.color = "#ffffffff"

				d.rel[0].to = d.rel[1].to = @rel
				d.rel[0].set_offset(@x, @y)

				d.auto_rel = true
				d.image = "#{@image}.png"
			end
		end

		ret
	end

	def setup_programs
		{"in" => "hover", "out" => "default"}.each do |sig, state|
			@c.program("#{@name}.#{sig}", :set_state) do |p|
				p.signal = "mouse,#{sig}"
				p.source = @name[0..-5] # cut '.led' suffix
				p.state = state
				p.mode = sig == "in" ? :accelerate : :decelerate
				p.time = 0.3
				p.targets << ProgramTarget.new(@part)
			end
		end
	end
end

class ToggleButton
	attr_reader :part

	def initialize(collection, name, rel, image, x, y, variable, signal)
		@col = collection
		@name = name
		@rel = rel
		@image = image
		@x = x
		@y = y
		@signal = signal
		@variable = variable

		@part = setup_part
		setup_programs
	end

	private
	def setup_part
		ret = nil

		@col.part(@name, :image) do |p|
			ret = p

			["inactive", "active"].each do |state|
				s = state == "inactive" ? "default" : state

				p.description(s) do |d|
					d.rel[0].to = d.rel[1].to = @rel
					d.rel[0].set_offset(@x, @y)

					d.auto_rel = true
					d.image = "#{@image}-#{state}.png"
				end
			end
		end

		ret
	end

	def setup_programs
		programs = []
		toggle_prg = nil
		emit_toggle_prg = nil

		@col.program("#{@name}.toggle", :exec_script) do |p|
			toggle_prg = p

			p.script =<<SMALL
new i = !get_int (#{@variable});
set_int (#{@variable}, i);

set_state (PART:"#{@name}", i ? "active" : "default", 0.0);
SMALL
		end

		@col.program("#{@name}.emit_toggle", :emit_signal) do |p|
			emit_toggle_prg = p

			p.emission_signal = @signal
			p.emission_source = @name
		end

		@col.program("#{@name}.on_toggle", :base) do |p|
			p.signal = @signal
			p.source = "Euphoria"
			p.after << ProgramAfter.new(toggle_prg)
		end

		@col.program("#{@name}.clicked", :base) do |p|
			p.signal = "mouse,clicked,1"
			p.source = @name
			p.after << ProgramAfter.new(toggle_prg)
			p.after << ProgramAfter.new(emit_toggle_prg)
		end
	end
end
