class VerticalSlider
	def initialize(c, name, rel, x, y, height = 95)
		background = nil
		clip = nil
		main = nil

		c.part("#{name}.background", :image) do |p|
			background = p

			p.description do |d|
				d.rel[0].to = d.rel[1].to = rel
				d.rel[0].set_offset(x, y)
				d.rel[1].set_rel(0.0, 0.0)
				d.rel[1].set_offset(x + 6, y + height)

				d.image = "background-fillbar-vertical.png"
			end
		end

		c.part("#{name}.clip", :rect) do |p|
			background.clip = p
			clip = p

			p.description.rel[1].to = background
		end

		c.part("#{name}.trough", :rect) do |p|
			p.description do |d|
				d.color = "#00000000"

				d.rel[0].to = d.rel[1].to = background
				d.rel[0].set_offset(3, -3)
				d.rel[1].set_offset(3, 3)
			end
		end

		c.part(name, :image) do |p|
			main = p
			clip.description.rel[0].to = p

			p.dragable.enabled[1] = true
			p.dragable.step[1] = 1
			p.dragable.count[1] = 1
			p.dragable.confine = c.part("#{name}.trough")

			["normal", "hover", "down"].each do |state|
				s = state == "normal" ? "default" : state
				p.description(s) do |d|
					d.set_size(15, 12)
					d.image = "slider-grip-#{state}.png"
				end
			end
		end

		c.part("#{name}.led", :image) do |p|
			p.repeat_events = true

			p.description do |d|
				d.visible = false
				d.color = "#ffffff00"

				d.rel[0].to = d.rel[1].to = main

				d.auto_rel = true
				d.image = "slider-grip-led.png"
			end

			p.description("visible") do |d|
				d.visible = true
				d.color = "#ffffffff"

				d.rel[0].to = d.rel[1].to = main

				d.auto_rel = true
				d.image = "slider-grip-led.png"
			end
		end

		c.program("#{name}.led.show", :set_state) do |p|
			p.signal ="mouse,down,1"
			p.source = main.name
			p.state = "visible"
			p.mode = :accelerate
			p.time = 0.5
			p.targets << ProgramTarget.new(c.part("#{name}.led"))
		end

		c.program("#{name}.led.hide", :set_state) do |p|
			p.signal = "mouse,up,1"
			p.source = main.name
			p.state = "default"
			p.mode = :decelerate
			p.time = 0.3
			p.targets << ProgramTarget.new(c.part("#{name}.led"))
		end
	end
end
